@@ -19,6 +19,9 @@ use warnings                                                   ;
 
 use XML::Smart::Shared qw( _unset_sig_warn _reset_sig_warn )   ;
 
+use vars qw($VERSION)                                          ;
+
+$VERSION = 0.01 ;
 ######################
 # _GENERATE_NULLTREE #
 ######################
@@ -34,7 +37,7 @@ sub _generate_nulltree {
     my @tree = @{$saver->{keyprev}} ;
     if (!@tree) { return ;}
     
-    if ( $I > 0 ) { push(@tree , "[$I]") ;}
+    if ( $I and $I > 0 ) { push(@tree , "[$I]") ;}
     
     my $tree = $saver->{tree} ;
     
